@@ -73,10 +73,10 @@ int KexiComboBoxBase::rowToHighlightForLookupTable() const
     bool ok;
     const int rowUid = origValue().toInt();
 //! @todo for now we're assuming the id is INTEGER
-    KexiTableViewData *tvData = popup()->tableView()->data();
+    KexiDB::TableViewData *tvData = popup()->tableView()->data();
     const int boundColumn = lookupFieldSchema->boundColumn();
     int row = -1;
-    for (KexiTableViewData::Iterator it(tvData->constBegin()); it != tvData->constEnd(); ++it) {
+    for (KexiDB::TableViewData::Iterator it(tvData->constBegin()); it != tvData->constEnd(); ++it) {
         row++;
         KexiDB::RecordData* record = *it;
         if (record->at(boundColumn).toInt(&ok) == rowUid && ok)
@@ -95,7 +95,7 @@ void KexiComboBoxBase::setValueInternal(const QVariant& add_, bool removeOld)
     m_updatePopupSelectionOnShow = true;
     QString add(add_.toString());
     if (add.isEmpty()) {
-        KexiTableViewData *relData = column() ? column()->relatedData() : 0;
+        KexiDB::TableViewData *relData = column() ? column()->relatedData() : 0;
         QVariant valueToSet;
         bool hasValueToSet = true;
         int rowToHighlight = -1;
@@ -172,11 +172,11 @@ KexiDB::RecordData* KexiComboBoxBase::selectItemForEnteredValueInLookupTable(con
 
     const bool valueIsText = v.type() == QVariant::String || v.type() == QVariant::CString; //most common case
     const QString txt(valueIsText ? v.toString().trimmed() : QString());
-    KexiTableViewData *lookupData = popup()->tableView()->data();
+    KexiDB::TableViewData *lookupData = popup()->tableView()->data();
     const int visibleColumn = lookupFieldSchema->visibleColumn(lookupData->columnsCount());
     if (-1 == visibleColumn)
         return 0;
-    KexiTableViewData::Iterator it(lookupData->constBegin());
+    KexiDB::TableViewData::Iterator it(lookupData->constBegin());
     int row;
     for (row = 0;it != lookupData->constEnd();++it, row++) {
         if (valueIsText) {
@@ -204,7 +204,7 @@ QString KexiComboBoxBase::valueForString(const QString& str, int* row,
         uint lookInColumn, uint returnFromColumn, bool allowNulls)
 {
     Q_UNUSED(returnFromColumn);
-    KexiTableViewData *relData = column() ? column()->relatedData() : 0;
+    KexiDB::TableViewData *relData = column() ? column()->relatedData() : 0;
     if (!relData)
         return QString(); //safety
     //use 'related table data' model
@@ -212,7 +212,7 @@ QString KexiComboBoxBase::valueForString(const QString& str, int* row,
     //.trimmed() is not generic!
 
     const QString txt(str.trimmed());
-    KexiTableViewData::Iterator it(relData->constBegin());
+    KexiDB::TableViewData::Iterator it(relData->constBegin());
     for (*row = 0;it != relData->constEnd();++it, (*row)++) {
         const QString s((*it)->at(lookInColumn).toString());
         if (s.trimmed().compare(txt, Qt::CaseInsensitive) == 0)
@@ -232,7 +232,7 @@ QString KexiComboBoxBase::valueForString(const QString& str, int* row,
 
 QVariant KexiComboBoxBase::value()
 {
-    KexiTableViewData *relData = column() ? column()->relatedData() : 0;
+    KexiDB::TableViewData *relData = column() ? column()->relatedData() : 0;
     KexiDB::LookupFieldSchema *lookupFieldSchema = 0;
     if (relData) {
         if (m_internalEditorValueChanged) {
@@ -300,7 +300,7 @@ void KexiComboBoxBase::clear()
 tristate KexiComboBoxBase::valueChangedInternal()
 {
     //avoid comparing values:
-    KexiTableViewData *relData = column() ? column()->relatedData() : 0;
+    KexiDB::TableViewData *relData = column() ? column()->relatedData() : 0;
     KexiDB::LookupFieldSchema *lookupFieldSchema = this->lookupFieldSchema();
     if (relData || lookupFieldSchema) {
         if (m_internalEditorValueChanged)
@@ -357,8 +357,8 @@ void KexiComboBoxBase::createPopup(bool show)
     if (!popup()) {
         setPopup(column() ? new KexiComboBoxPopup(thisWidget, *column())
                  : new KexiComboBoxPopup(thisWidget, *field()));
-        QObject::connect(popup(), SIGNAL(rowAccepted(KexiDB::RecordData*, int)),
-                         thisWidget, SLOT(slotRowAccepted(KexiDB::RecordData*, int)));
+        QObject::connect(popup(), SIGNAL(rowAccepted(KexiDB::RecordData*,int)),
+                         thisWidget, SLOT(slotRowAccepted(KexiDB::RecordData*,int)));
         QObject::connect(popup()->tableView(), SIGNAL(itemSelected(KexiDB::RecordData*)),
                          thisWidget, SLOT(slotItemSelected(KexiDB::RecordData*)));
 
@@ -393,7 +393,7 @@ void KexiComboBoxBase::createPopup(bool show)
         if (m_updatePopupSelectionOnShow) {
             int rowToHighlight = -1;
             KexiDB::LookupFieldSchema *lookupFieldSchema = this->lookupFieldSchema();
-            KexiTableViewData *relData = column() ? column()->relatedData() : 0;
+            KexiDB::TableViewData *relData = column() ? column()->relatedData() : 0;
             if (lookupFieldSchema) {
                 rowToHighlight = rowToHighlightForLookupTable();
             } else if (relData) {
@@ -463,7 +463,7 @@ void KexiComboBoxBase::slotItemSelected(KexiDB::RecordData*)
     //kDebug(44010) << "m_visibleValue=" << m_visibleValue;
 
     QVariant valueToSet;
-    KexiTableViewData *relData = column() ? column()->relatedData() : 0;
+    KexiDB::TableViewData *relData = column() ? column()->relatedData() : 0;
     KexiDB::LookupFieldSchema *lookupFieldSchema = this->lookupFieldSchema();
 
     m_visibleValue = lookupFieldSchema ? visibleValueForLookupField() : QVariant();
