@@ -50,12 +50,14 @@
 struct KisNodeModel::Private
 {
 public:
-    Private() : indexConverter(0),
+    Private() : shapeController(0),
+                indexConverter(0),
                 dummiesFacade(0),
                 needFinishRemoveRows(false),
                 needFinishInsertRows(false) {}
 
     KisImageWSP image;
+    KisShapeController *shapeController;
     bool showRootLayer;
     QList<KisNodeDummy*> updateQueue;
     QTimer* updateTimer;
@@ -140,8 +142,8 @@ void KisNodeModel::connectDummy(KisNodeDummy *dummy, bool needConnect)
     KisNodeProgressProxy *progressProxy = node->nodeProgressProxy();
     if(progressProxy) {
         if(needConnect) {
-            connect(progressProxy, SIGNAL(percentageChanged(int, const KisNodeSP&)),
-                    SLOT(progressPercentageChanged(int, const KisNodeSP&)));
+            connect(progressProxy, SIGNAL(percentageChanged(int,KisNodeSP)),
+                    SLOT(progressPercentageChanged(int,KisNodeSP)));
         } else {
             progressProxy->disconnect(this);
         }
@@ -159,9 +161,10 @@ void KisNodeModel::connectDummies(KisNodeDummy *dummy, bool needConnect)
     }
 }
 
-void KisNodeModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade, KisImageWSP image)
+void KisNodeModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade, KisImageWSP image, KisShapeController *shapeController)
 {
     m_d->image = image;
+    m_d->shapeController = shapeController;
 
     if(m_d->dummiesFacade) {
         m_d->dummiesFacade->disconnect(this);
@@ -177,8 +180,8 @@ void KisNodeModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade, KisImag
             connectDummies(rootDummy, true);
         }
 
-        connect(m_d->dummiesFacade, SIGNAL(sigBeginInsertDummy(KisNodeDummy*, int, const QString&)),
-                SLOT(slotBeginInsertDummy(KisNodeDummy*, int, QString)));
+        connect(m_d->dummiesFacade, SIGNAL(sigBeginInsertDummy(KisNodeDummy*,int,QString)),
+                SLOT(slotBeginInsertDummy(KisNodeDummy*,int,QString)));
         connect(m_d->dummiesFacade, SIGNAL(sigEndInsertDummy(KisNodeDummy*)),
                 SLOT(slotEndInsertDummy(KisNodeDummy*)));
         connect(m_d->dummiesFacade, SIGNAL(sigBeginRemoveDummy(KisNodeDummy*)),
@@ -308,7 +311,7 @@ QModelIndex KisNodeModel::parent(const QModelIndex &index) const
 
 QVariant KisNodeModel::data(const QModelIndex &index, int role) const
 {
-    if(!m_d->dummiesFacade || !index.isValid()) return QVariant();
+    if (!m_d->dummiesFacade || !index.isValid() || !m_d->image.isValid()) return QVariant();
 
     KisNodeSP node = nodeFromIndex(index);
 
@@ -366,7 +369,7 @@ bool KisNodeModel::setData(const QModelIndex &index, const QVariant &value, int
             // don't record undo/redo for visibility, locked or alpha locked changes
             PropertyList proplist = value.value<PropertyList>();
             bool undo = true;
-            foreach(KoDocumentSectionModel::Property prop, proplist) {
+            foreach(const KoDocumentSectionModel::Property &prop, proplist) {
                 if (prop.name == i18n("Visible") && node->visible() !=prop.state.toBool()) undo = false;
                 if (prop.name == i18n("Locked") && node->userLocked() != prop.state.toBool()) undo = false;
                 if (prop.name == i18n("Active")) {
@@ -422,6 +425,7 @@ QStringList KisNodeModel::mimeTypes() const
 {
     QStringList types;
     types << QLatin1String("application/x-krita-node");
+    types << QLatin1String("application/x-qt-image");
     return types;
 }
 
@@ -457,11 +461,32 @@ bool KisNodeModel::dropMimeData(const QMimeData * data, Qt::DropAction action, i
 {
     Q_UNUSED(column);
 
-    const KisMimeData *mimedata = qobject_cast<const KisMimeData*>(data);
-    KisNodeSP node = mimedata ? mimedata->node() : 0;
+    bool copyNode = action == Qt::CopyAction;
+    KisNodeSP node =
+        KisMimeData::tryLoadInternalNode(data,
+                                         m_d->image,
+                                         m_d->shapeController,
+                                         copyNode /* IN-OUT */);
+
+    if (!node) {
+        QRect imageBounds = m_d->image->bounds();
+        node = KisMimeData::loadNode(data,
+                                     imageBounds, imageBounds.center(),
+                                     false,
+                                     m_d->image, m_d->shapeController);
+        copyNode = true;
+    }
 
     if (!node) return false;
 
+    if (copyNode) {
+        /**
+         * Don't try to move a node originating from another image,
+         * just copy it.
+         */
+        action = Qt::CopyAction;
+    }
+
     KisNodeDummy *parentDummy = 0;
     KisNodeDummy *aboveThisDummy = 0;
 
@@ -487,9 +512,10 @@ bool KisNodeModel::dropMimeData(const QMimeData * data, Qt::DropAction action, i
     bool result = true;
 
     if (action == Qt::CopyAction) {
-        emit requestAddNode(node->clone(), parentDummy->node(), aboveThisNode);
+        emit requestAddNode(node, parentDummy->node(), aboveThisNode);
     }
     else if (action == Qt::MoveAction) {
+        Q_ASSERT(node->graphListener() == m_d->image.data());
         emit requestMoveNode(node, parentDummy->node(), aboveThisNode);
     }
     else {
