@@ -352,10 +352,11 @@ void KisOpenGLImageTextures::getTextureSize(KisGLTexturesInfo *texturesInfo)
 
 void KisOpenGLImageTextures::updateTextureFormat()
 {
-    m_texturesInfo.format = GL_RGBA8;
+    m_texturesInfo.internalFormat = GL_RGBA8;
     m_texturesInfo.type = GL_UNSIGNED_BYTE;
+    m_texturesInfo.format = GL_BGRA;
 
-    #ifdef HAVE_GLEW
+#ifdef HAVE_GLEW
 
     KoID colorModelId = m_image->colorSpace()->colorModelId();
     KoID colorDepthId = m_image->colorSpace()->colorDepthId();
@@ -366,11 +367,11 @@ void KisOpenGLImageTextures::updateTextureFormat()
         if (colorDepthId == Float16BitsColorDepthID) {
 
             if (GLEW_ARB_texture_float) {
-                m_texturesInfo.format = GL_RGBA16F_ARB;
+                m_texturesInfo.internalFormat = GL_RGBA16F_ARB;
                 dbgUI << "Using ARB half";
             }
             else if (GLEW_ATI_texture_float){
-                m_texturesInfo.format = GL_RGBA_FLOAT16_ATI;
+                m_texturesInfo.internalFormat = GL_RGBA_FLOAT16_ATI;
                 dbgUI << "Using ATI half";
             }
             else if (GLEW_ARB_half_float_pixel) {
@@ -380,23 +381,25 @@ void KisOpenGLImageTextures::updateTextureFormat()
                 dbgUI << "Pixel type float";
                 m_texturesInfo.type = GL_FLOAT;
             }
+            m_texturesInfo.format = GL_RGBA;
         }
         else if (colorDepthId == Float32BitsColorDepthID) {
 
             if (GLEW_ARB_texture_float) {
-                m_texturesInfo.format = GL_RGBA32F_ARB;
+                m_texturesInfo.internalFormat = GL_RGBA32F_ARB;
                 dbgUI << "Using ARB float";
                 m_texturesInfo.type = GL_FLOAT;
             }
             else if (GLEW_ATI_texture_float) {
-                m_texturesInfo.format = GL_RGBA_FLOAT32_ATI;
+                m_texturesInfo.internalFormat = GL_RGBA_FLOAT32_ATI;
                 dbgUI << "Using ATI float";
                 m_texturesInfo.type = GL_FLOAT;
             }
+            m_texturesInfo.format = GL_RGBA;
         }
         else if (colorDepthId == Integer16BitsColorDepthID) {
             dbgUI << "Using 16 bits rgba";
-            m_texturesInfo.format = GL_RGBA16;
+            m_texturesInfo.internalFormat = GL_RGBA16;
             m_texturesInfo.type = GL_UNSIGNED_SHORT;
         }
     }
@@ -404,7 +407,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
         // We will convert the colorspace to 16 bits rgba, instead of 8 bits
         if (colorDepthId == Integer16BitsColorDepthID) {
             dbgUI << "Using conversion to 16 bits rgba";
-            m_texturesInfo.format = GL_RGBA16;
+            m_texturesInfo.internalFormat = GL_RGBA16;
             m_texturesInfo.type = GL_UNSIGNED_SHORT;
         }
     }
