@@ -30,8 +30,6 @@
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
 #include <klocale.h>
-#include <kservice.h>
-#include <kservicetypetrader.h>
 #include <kglobal.h>
 
 #include "KoPluginLoader.h"
@@ -103,13 +101,13 @@ void KoColorSpaceRegistry::init()
     config.whiteList = "ColorSpacePlugins";
     config.blacklist = "ColorSpacePluginsDisabled";
     config.group = "calligra";
-    KoPluginLoader::instance()->load("Calligra/ColorSpace", "[X-Pigment-MinVersion] <= 4", config);
+    KoPluginLoader::instance()->load("Calligra/ColorSpace", "[X-Pigment-PluginVersion] == 27", config);
 
     KoPluginLoader::PluginsConfig configExtensions;
     configExtensions.whiteList = "ColorSpaceExtensionsPlugins";
     configExtensions.blacklist = "ColorSpaceExtensionsPluginsDisabled";
     configExtensions.group = "calligra";
-    KoPluginLoader::instance()->load("Calligra/ColorSpaceExtension", "[X-Pigment-MinVersion] <= 4", configExtensions);
+    KoPluginLoader::instance()->load("Calligra/ColorSpaceExtension", "[X-Pigment-PluginVersion] == 27", configExtensions);
 
 
     dbgPigment << "Loaded the following colorspaces:";
@@ -384,7 +382,6 @@ const KoColorSpace * KoColorSpaceRegistry::colorSpace(const QString &csID, const
                 dbgPigmentCSRegistry << "Unknown color space type :" << csf;
                 return 0;
             }
-            Q_ASSERT(csf->profileIsCompatible(profile));
             if (!csf->profileIsCompatible(profile ) ) {
                 return 0;
             }
