@@ -30,8 +30,8 @@
 #include "Region.h"
 #include "Sheet.h"
 
-#include <KDebug>
-#include <KMessageBox>
+#include <kdebug.h>
+#include <kmessagebox.h>
 
 using namespace Calligra::Sheets;
 
